@@ -7,11 +7,12 @@ import subprocess as sp
 import pytest
 
 from snakemake_interface_software_deployment_plugins import (
-    ArchiveableEnvBase,
+    CacheableEnvBase,
     DeployableEnvBase,
     EnvBase,
     EnvSpecBase,
     EnvSpecSourceFile,
+    PinnableEnvBase,
     SoftwareReport,
 )
 from snakemake_interface_software_deployment_plugins.settings import (
@@ -51,10 +52,22 @@ class TestSoftwareDeploymentBase(ABC):
         ...
 
     @abstractmethod
-    def get_software_deployment_provider_settings(
+    def get_settings(
         self,
     ) -> Optional[SoftwareDeploymentSettingsBase]: ...
 
+    @abstractmethod
+    def get_settings_cls(self) -> Optional[Type[SoftwareDeploymentSettingsBase]]: ...
+
+    def test_envspec_str(self):
+        print("env spec", str(self.get_env_spec()))
+
+    def test_default_settings(self):
+        settings_cls = self.get_settings_cls()
+        if settings_cls is None:
+            pytest.skip("No settings class defined.")
+        settings_cls()
+
     def test_shellcmd(self, tmp_path):
         env = self._get_env(tmp_path)
 
@@ -77,15 +90,30 @@ class TestSoftwareDeploymentBase(ABC):
         cmd = env.managed_decorate_shellcmd(self.get_test_cmd())
         assert sp.run(cmd, shell=True, executable=self.shell_executable).returncode == 0
 
-    def test_archive(self, tmp_path):
+    def test_cache(self, tmp_path):
         env = self._get_env(tmp_path)
-        if not isinstance(env, ArchiveableEnvBase):
-            pytest.skip("Environment either not deployable or not archiveable.")
+        if not env.is_cacheable():
+            pytest.skip("Environment either not deployable or not cacheable.")
+
+        assert isinstance(env, CacheableEnvBase)
+
+        asyncio.run(env.cache_assets())
 
         self._deploy(env, tmp_path)
 
-        asyncio.run(env.archive())
-        assert any((tmp_path / "archives").iterdir())
+        assert any(env.cache_path.iterdir())
+
+    def test_pin(self, tmp_path):
+        env = self._get_env(tmp_path)
+        if not env.is_pinnable():
+            pytest.skip("Environment is not pinnable.")
+
+        assert isinstance(env, PinnableEnvBase)
+
+        asyncio.run(env.pin())
+        assert env.pinfile.exists()
+        print("Pinfile content:", env.pinfile.read_text(), sep="\n")
+        self._deploy(env, tmp_path)
 
     def test_report_software(self, tmp_path):
         env = self._get_env(tmp_path)
@@ -101,10 +129,15 @@ class TestSoftwareDeploymentBase(ABC):
 
     def test_source_path_attributes(self):
         spec = self.get_env_spec()
+
+        def is_source_file_or_none(attr: str) -> bool:
+            val = getattr(spec, attr)
+            return val is None or isinstance(val, EnvSpecSourceFile)
+
         assert all(
             isinstance(attr, str)
             and hasattr(spec, attr)
-            and isinstance(getattr(spec, attr), (EnvSpecSourceFile, None))
+            and is_source_file_or_none(attr)
             for attr in spec.source_path_attributes()
         ), "bug in plugin: all source path attributes must be of type EnvSpecSourceFile"
 
@@ -112,24 +145,38 @@ class TestSoftwareDeploymentBase(ABC):
         spec = deepcopy(self.get_env_spec())
         for attr in spec.source_path_attributes():
             source_file = getattr(spec, attr)
-            source_file.cached = source_file.path_or_uri
+            if source_file is not None:
+                source_file.cached = source_file.path_or_uri
         return spec
 
     def _get_env(self, tmp_path) -> EnvBase:
         env_cls = self.get_env_cls()
         spec = self._get_cached_env_spec()
-        args = {"settings": self.get_software_deployment_provider_settings()}
-        if issubclass(env_cls, DeployableEnvBase):
-            args["deployment_prefix"] = tmp_path / "deployments"
-        if issubclass(env_cls, ArchiveableEnvBase):
-            args["archive_prefix"] = tmp_path / "archives"
+
+        tempdir = tmp_path / "temp"
+        deployment_prefix = tmp_path / "deployments"
+        cache_prefix = tmp_path / "cache"
+        pinfile_prefix = tmp_path / "pinfiles"
+        tempdir.mkdir(parents=True, exist_ok=True)
+        deployment_prefix.mkdir(parents=True, exist_ok=True)
+        cache_prefix.mkdir(parents=True, exist_ok=True)
+        pinfile_prefix.mkdir(parents=True, exist_ok=True)
+
         return env_cls(
-            spec=spec, within=None, shell_executable=self.shell_executable, **args
+            spec=spec,
+            within=None,
+            settings=self.get_settings(),
+            shell_executable=self.shell_executable,
+            tempdir=tempdir,
+            deployment_prefix=deployment_prefix,
+            cache_prefix=cache_prefix,
+            pinfile_prefix=pinfile_prefix,
         )
 
-    def _deploy(self, env: DeployableEnvBase, tmp_path):
-        if not isinstance(env, DeployableEnvBase):
+    def _deploy(self, env: EnvBase, tmp_path):
+        if not env.is_deployable():
             pytest.skip("Environment is not deployable.")
 
+        assert isinstance(env, DeployableEnvBase)
         asyncio.run(env.deploy())
         assert any((tmp_path / "deployments").iterdir())
